/*
 * Copyright 2013 (c) Anna Schumaker.
 */
#ifndef OCARINA_DATABASE_H
#define OCARINA_DATABASE_H

#include <file.h>
#include <print.h>

#include <map>
#include <set>
#include <vector>


class DatabaseEntry {
public:
	std::string primary_key;
	bool valid;

	DatabaseEntry();
	virtual void write(File &) = 0;
	virtual void read(File &) = 0;
#ifdef CONFIG_TEST
	virtual void print() = 0;
#endif /* CONFIG_TEST */
};


class IndexEntry : public DatabaseEntry {
public:
	std::set<unsigned int> values;

	IndexEntry();
	IndexEntry(const std::string &, unsigned int);
	~IndexEntry();
	void write(File &);
	void read(File &);
#ifdef CONFIG_TEST
	void print();
#endif /* CONFIG_TEST */
	void insert(unsigned int);
	void remove(unsigned int);
};


template <class T>
class Database {
private:
	std::vector<T> db;
	std::map<const std::string, unsigned int> keys;
	unsigned int _size;
	File file;

public:
	Database(std::string);
	~Database();
	void save();
	void load();
#ifdef CONFIG_TEST
	void clear();
	void print();
	void print_keys();
#endif /* CONFIG_TEST */

	unsigned int insert(T);
	void remove(unsigned int);
	unsigned int size();
	unsigned int num_rows();

	unsigned int first();
	unsigned int last();
	unsigned int next(unsigned int);
	bool has_key(const std::string &);
	unsigned int find_index(const std::string &);
	T &find(const std::string &);
	T &operator[](unsigned int);
};

static inline void index_insert(Database<IndexEntry> &db,
				const std::string &key,
				unsigned int val)
{
	try {
		db.find(key).insert(val);
	} catch (int error) {
		db.insert(IndexEntry(key, val));
	}
}

static inline void index_remove(Database<IndexEntry> &db,
				const std::string &key,
				unsigned int val)
{
	unsigned int i = db.find_index(key);
	db[i].remove(val);
	if (db[i].values.size() == 0)
		db.remove(db.find_index(key));
}

#include "database.hpp"

#endif /* OCARINA_DATABASE_H */
